@@ -5,7 +5,9 @@
 #include "pytype_addresses.h"
 #include "pytype_basics.h"
 #include "structmember.h"
+#include "python_compatibility.h"
 #include <pi-buffer.h>
+#include "python_compatibility.h"
 
 /*******************************************************
  * Create and delloc methods for objects 
@@ -30,12 +32,13 @@ extern PyObject* PyPiAddress_New(PyTypeObject *type, PyObject *args, PyObject *k
 extern int PyPiAddress_Init(PyObject *self, PyObject *args, PyObject *kwds) {
   PyPiAddress* fromaddress = NULL;
   PyPiAddress* address = NULL;
+  PyObject* filters = NULL;
   int i;
   int malloc_failed = 0;
   
-  static char *kwlist[] = {"address", NULL};
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O", kwlist, 
-				   &fromaddress)) {
+  static char *kwlist[] = {"address","record_field_filters", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OO", kwlist, 
+				   &fromaddress, &filters)) {
     return -1;
   }
   
@@ -119,14 +122,58 @@ static PyObject * PyPiAddress_Allocate(PyTypeObject *type, int nitems) {
   }
 }
 
+/**
+ * Wrap an existing Address in a jppy address. This makes
+ * a copy of a into a newly allocated struct inside the python object.
+ *
+ * @param a the palm object
+ * @param rt
+ * @param unique_id
+ * @param attrib
+ * @param size
+ * @param buf
+ * @return a newly created python event object that represents the data in a
+ */
 extern PyObject* PyPiAddress_Wrap(struct Address *a, PCRecType rt, 
 				 unsigned int unique_id, unsigned char attrib,
-				 int size, void* buf) {
+				 int size, void* buf, PyObject *filters) {
   PyPiAddress* address;
   int i;
   int malloc_failed = 0;
   
-  address = (PyPiAddress*)PyPiAddress_New(&AddressType,NULL,NULL);
+
+ PyObject *python_mod, *python_mdict, *address_class, *python_args, *python_kw;
+  python_mod = PyImport_Import(PyString_FromString("jppy.jpilot.legacy"));
+  if (python_mod == NULL) {
+    PyErr_Print();
+    return NULL;
+  }
+  python_mdict = PyModule_GetDict(python_mod);
+  if (python_mdict == NULL) {
+    PyErr_Print();
+    Py_DECREF(python_mod);
+    return NULL;
+  }  
+  Py_INCREF(python_mdict);
+  Py_DECREF(python_mod);
+  address_class = PyDict_GetItemString(python_mdict, "Address"); /* borrowed reference */
+  if (address_class == NULL) {
+    PyErr_Print();
+    Py_DECREF(python_mdict);
+    return NULL;
+  }
+  Py_INCREF(address_class);
+  python_args = Py_BuildValue("()");
+  python_kw = Py_BuildValue("{s:O}","record_field_filters",filters);
+  address = (PyPiAddress*) PyObject_Call(address_class, python_args, python_kw);
+  Py_DECREF(address_class);
+  Py_DECREF(python_args);
+  Py_DECREF(python_kw);
+  if (address == NULL) {
+    PyErr_Print();
+    return NULL;
+  }
+  Py_INCREF(address);
 
   /* copy birthday stuff, phoneLabels and showPhone */
   memcpy(&(address->a), a, sizeof(struct Address));
@@ -148,8 +195,8 @@ extern PyObject* PyPiAddress_Wrap(struct Address *a, PCRecType rt,
 
   if (malloc_failed) { /* Oh well, undo the mallocs that worked */
     for(i=0;i<19;i++) {
-      if (a->entry[i]) {
-	free(a->entry[i]);
+      if (address->a.entry[i]) {
+	free(address->a.entry[i]);
       }
     }
     PyErr_SetString(PyExc_MemoryError,"Unable to allocate memory for address entires");
@@ -161,6 +208,10 @@ extern PyObject* PyPiAddress_Wrap(struct Address *a, PCRecType rt,
 
 static void PyPiAddress_Dealloc(PyPiAddress* self) {
   free_Address(&(self->a));
+  if (self->filters != NULL) {
+    Py_DECREF(self->filters);
+    self->filters = NULL;
+  }
   if (self->saved_br.size > 0 && self->saved_br.buf) {
     free(self->saved_br.buf);
   }
@@ -230,6 +281,7 @@ static PyObject* PyPiAddress_keys(PyObject* self) {
     PyList_Append(list, value);
     Py_DECREF(value);
   }
+  PyPi_extend_keys_from_filters((PyPiBase*)self, list);
   return list;
 }
 
@@ -434,7 +486,7 @@ static PyMethodDef PyPiAddress_Methods[] = {
   { "keys", (PyCFunction)PyPiAddress_keys, METH_NOARGS, "Return a list of available keys"},
   { "items",(PyCFunction)PyPiAddress_items, METH_NOARGS, "Return a list of available items"},
   { "values",(PyCFunction)PyPiAddress_values, METH_NOARGS, "Return a list of available items"},
-  {NULL,NULL}
+  {NULL,NULL,0,NULL}
 };
 
 
@@ -449,13 +501,19 @@ static PyGetSetDef PyPiAddress_Getseters[] = {
 };
 
 /**** mapping interface ****/
-int PyPiAddress_Len(self) {
-  return 21;
+int PyPiAddress_Len(PyObject* self) {
+  int len=0;
+  // quite expensive way to do it, but we need to get filters too
+  PyObject *keys = PyPiAddress_keys(self);
+  len = PySequence_Size(keys);
+  Py_DECREF(keys);
+  return len;
 }
 
 
 PyObject *PyPiAddress_GetItem(PyPiAddress* self,  PyObject* key) {
   char *keystring;
+  PyObject *result;
   PyObject* py_list;
   int n, i;
 
@@ -464,6 +522,12 @@ PyObject *PyPiAddress_GetItem(PyPiAddress* self,  PyObject* key) {
     return Py_None;  
   }
 
+  if ((result = PyPi_GetItem_from_filters((PyPiBase *)self, key)) != NULL)
+    return result;
+  else if (PyErr_Occurred() != NULL)
+    return NULL;
+
+
   Py_INCREF(key);
   keystring = PyString_AsString(key);
 
@@ -562,7 +626,12 @@ int PyPiAddress_SetItem(PyPiAddress* self, PyObject* key, PyObject* value) {
     PyErr_SetString(PyExc_TypeError,"key must be a String");
     return -1;
   }
-  
+
+  if (PyPi_SetItem_from_filters((PyPiBase *)self, key, value) > 0)
+    return 0;
+  else if (PyErr_Occurred() != NULL)
+    return -1;
+
   Py_INCREF(key);
   keystring = PyString_AsString(key);
 
@@ -661,7 +730,7 @@ int PyPiAddress_SetItem(PyPiAddress* self, PyObject* key, PyObject* value) {
 }
 
 static PyMappingMethods PyPiAddress_Mapping = {
-  (inquiry)PyPiAddress_Len,
+  (lenfunc)PyPiAddress_Len,
   (binaryfunc)PyPiAddress_GetItem, 
   (objobjargproc)PyPiAddress_SetItem,
 };
